@@ -18,9 +18,10 @@
 
 import datetime
 
+from keystone.common import manager
 from keystone import config
 from keystone import exception
-from keystone.common import manager
+from keystone.openstack.common import timeutils
 
 
 CONF = config.CONF
@@ -38,8 +39,13 @@ class Manager(manager.Manager):
     def __init__(self):
         super(Manager, self).__init__(CONF.token.driver)
 
-    def revoke_tokens(self, context, user_id):
-        for token_id in self.list_tokens(context, user_id):
+    def revoke_tokens(self, context, user_id, tenant_id=None):
+        """Invalidates all tokens held by a user (optionally for a tenant).
+
+        If a specific tenant ID is not provided, *all* tokens held by user will
+        be revoked.
+        """
+        for token_id in self.list_tokens(context, user_id, tenant_id):
             self.delete_token(context, token_id)
 
 
@@ -101,10 +107,19 @@ class Driver(object):
         """
         raise exception.NotImplemented()
 
-    def revoke_tokens(self, user_id):
-        """Invalidates all tokens held by a user.
+    def list_revoked_tokens(self):
+        """Returns a list of all revoked tokens
 
-        :raises: keystone.exception.UserNotFound
+        :returns: list of token_id's
+
+        """
+        raise exception.NotImplemented()
+
+    def revoke_tokens(self, user_id, tenant_id=None):
+        """Invalidates all tokens held by a user (optionally for a tenant).
+
+        :raises: keystone.exception.UserNotFound,
+                 keystone.exception.TenantNotFound
         """
         raise exception.NotImplemented()
 
@@ -115,4 +130,4 @@ class Driver(object):
 
         """
         expire_delta = datetime.timedelta(seconds=CONF.token.expiration)
-        return datetime.datetime.utcnow() + expire_delta
+        return timeutils.utcnow() + expire_delta
